/* 
*
*  ProLinga-Repository
*
*  Copyright (C) 2002-2008 Xobas Software.
*  All rights reserved.
*  
*  This file is part of ProLinga-Repository.
*
*  ProLinga-Repository is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  ProLinga-Repository is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with ProLinga-Repository.  If not, see <http://www.gnu.org/licenses/>.
*
*  More information is available at the following addresses:
*
*  Website     : http://www.prolinga.org
*
*  Email       : prolinga-list@prolinga.org
*
*
*/

#include "RepCommon.h"
#include <dbxml/DbXml.hpp>
#include "RepConfig.hpp"

using namespace DbXml;

void Usage(const char *progName)
{
	printf("Usage : %s [ [--version] || [--config file_name] repository_name ] \n", progName);
}

int main(int argc, char *argv[])
{
	char *systemPath;
	char *configFileName = DEF_CONFIG_FILE;
	char *repositoryName = NULL;
	ConfigRepPtr confRepPtr;
	int i;
	DbEnv *env;
	XmlManager mgr;
	XmlContainer container;
	XmlUpdateContext uc;

	/* Process command line arguments */
	if (argc <= 1)
	{
		Usage(argv[0]);
		return 0;
	}

	for (i=1;i<argc;i++)
	{
		if ((!strcmp(argv[i], "-version")) || (!strcmp(argv[i], "--version")))
		{
			fprintf(stderr,"%s version: %s\n", argv[0], VERSION);
			fprintf(stderr,"Configuration file: %s\n", DEF_CONFIG_FILE);
			fprintf(stderr,"Copyright © 2002-2008 The ProLinga Team.\n");
			fprintf(stderr,"All rights reserved.\n");
			return 0;
		}
		else if ((!strcmp(argv[i], "-config")) || (!strcmp(argv[i], "--config")))
		{
			if ((i+1) >= argc)
			{
				Usage(argv[0]);
				return 0;
			}
			i++;
			configFileName = argv[i];
		}
		else
		{
			if (repositoryName != NULL)
			{
				Usage(argv[0]);
				return 0;
			}
			repositoryName = argv[i];
		}
	}	
	/* Return if no repository name given */
	if (repositoryName == NULL)
	{
		Usage(argv[0]);
		return 0;
	}

	/* Get Config */
	confRepPtr = new ConfigRep(configFileName);
			
	try
	{
	        /* Set DB environment */
	        env = new DbEnv(0);

		/* Create Repository if not present */
		systemPath = confRepPtr->getStringValue("SystemPath", DEF_SYSTEM_PATH);
	        env->open(systemPath, DB_CREATE|DB_INIT_MPOOL, 0);
		mgr = XmlManager(env, DBXML_ADOPT_DBENV);

		mgr.setDefaultContainerType(XmlContainer::WholedocContainer);
		container = mgr.createContainer(repositoryName);
		uc = mgr.createUpdateContext();	

		/* Declare index for application name */
		container.addIndex(
			"",
			"Application",
			"node-attribute-equality-string",
			uc);

		container.addIndex(
			"",
			"Application",
			"node-attribute-substring-string",
			uc);

		/* Declare index for object type */
		container.addIndex(
			"",
			"Type",
			"node-attribute-equality-string",
			uc);

		container.addIndex(
			"",
			"Type",
			"node-attribute-substring-string",
			uc);

		/* Declare index for object name */
		container.addIndex(
			"",
			"Name",
			"node-attribute-equality-string",
			uc);

		container.addIndex(
			"",
			"Name",
			"node-attribute-substring-string",
			uc);

		/* Flush Repository */
		container.sync();

		/* Clean up */
		delete confRepPtr;
	}

	catch(DbException &e)
	{
		std::cerr << e.what() << "\n";
	}

	catch(std::exception &e)
	{
		std::cerr << e.what() << "\n";
	}
	return 0;

}

